@@ -157,6 +157,14 @@ frailtyMMpen <- function(formula, data, frailty = "gamma", power = NULL, penalty
       d = m[[1]][, 2]
       a = N
       
+      neworder = order(y, decreasing = TRUE)
+      newrank = seq(1, N, 1)[order(neworder)]
+      
+      y = y[neworder]
+      X = X[neworder, ]
+      d = d[neworder]
+      newid = newid[neworder]
+      
     }
     
     if (length(cluster_id) == 1) {
@@ -191,6 +199,14 @@ frailtyMMpen <- function(formula, data, frailty = "gamma", power = NULL, penalty
       d = m[[1]][nord, 2]
       a = max(newid) + 1
       
+      neworder = order(y, decreasing = TRUE)
+      newrank = seq(1, N, 1)[order(neworder)]
+      
+      y = y[neworder]
+      X = X[neworder, ]
+      d = d[neworder]
+      newid = newid[neworder]
+      
     }
     
     
@@ -270,11 +286,11 @@ frailtyMMpen <- function(formula, data, frailty = "gamma", power = NULL, penalty
     
     initGam = frailtyMMcal(y, X, d, N, a, newid, frailty = "Gamma", maxit = 10, threshold = threshold, type = 1)
     
-    # ini = frailtyMMcal(y, X, d, N, a, newid,
-    #                    coef.ini = initGam$coef, est.tht.ini = initGam$est.tht, lambda.ini = initGam$lambda,
-    #                    frailty = frailty, power = power, penalty = NULL, maxit = maxit, threshold = tol, type = 1)
+    if (sum(abs(initGam$coef) > 1e-6) == 0) {
+      initGam = frailtyMMcal(y, X, d, N, a, newid, frailty = frailty, maxit = 10, threshold = threshold, type = 1)
+    }
+
     ini = initGam
-    
     coef0 = ini$coef
     est.tht0 = ini$est.tht
     lambda0 = ini$lambda
@@ -286,6 +302,9 @@ frailtyMMpen <- function(formula, data, frailty = "gamma", power = NULL, penalty
     likelihood_all = list()
     BIC_all = list()
     
+    width <- options()$width/2
+    len_tune = length(tuneseq)
+    
     for (z in seq_len(length(tuneseq))) {
       cur = frailtyMMcal(y, X, d, N, a, newid,
                          coef.ini = coef0, est.tht.ini = est.tht0, lambda.ini = lambda0, safe.ini = list(coef = ini$coef, est.tht = ini$est.tht, lambda = ini$lambda),
@@ -304,9 +323,12 @@ frailtyMMpen <- function(formula, data, frailty = "gamma", power = NULL, penalty
       likelihood_all[[z]] = likelihood0
       BIC_all[[z]] = -2*likelihood0 + max(1, log(log(p + 1)))*(sum(abs(coef0) > threshold) + 1)*log(N)
       
+      progress_width = floor(z/len_tune*width)
+      cat('[', paste0(c(rep('=', progress_width), '>', rep('-', width - progress_width)),  collapse=''), ']', round(z/len_tune*100),'%\r')
       
       if (sum(abs(coef0)) < threshold) {
-        cat(sum(abs(coef0)), "????\n")
+        progress_width = width
+        cat('[', paste0(c(rep('=', progress_width), '>', rep('-', width - progress_width)),  collapse=''), ']', round(100),'%\r')
         break
       }
       
@@ -315,8 +337,6 @@ frailtyMMpen <- function(formula, data, frailty = "gamma", power = NULL, penalty
         est.tht0 = ini$est.tht
         lambda0 = ini$lambda
       } 
-      
-      cat(z, "---------\n")
     }
     
     
@@ -349,12 +369,11 @@ frailtyMMpen <- function(formula, data, frailty = "gamma", power = NULL, penalty
     
     initGam = frailtyMMcal(y, X, d, N, b, NULL, frailty = "Gamma", power = NULL, penalty = NULL, maxit = 10, threshold = tol, type = 2)
     
-    # ini =  frailtyMMcal(y, X, d, N, b, NULL,
-    #                     coef.ini = initGam$coef, est.tht.ini = initGam$est.tht, lambda.ini = initGam$lambda,
-    #                     frailty = frailty, power = power, penalty = NULL, maxit = maxit, threshold = tol, type = 2)
+    if (sum(abs(initGam$coef) > 1e-6) == 0) {
+      initGam = frailtyMMcal(y, X, d, N, b, NULL, frailty = frailty, power = NULL, penalty = NULL, maxit = 10, threshold = tol, type = 2)
+    }
     
     ini = initGam
-    
     coef0 = ini$coef
     est.tht0 = ini$est.tht
     lambda0 = ini$lambda
@@ -366,6 +385,9 @@ frailtyMMpen <- function(formula, data, frailty = "gamma", power = NULL, penalty
     likelihood_all = list()
     BIC_all = list()
     
+    width <- options()$width/2
+    len_tune = length(tuneseq)
+    
     for (z in seq_len(length(tuneseq))) {
       cur = frailtyMMcal(y, X, d, N, b, NULL,
                          coef.ini = coef0, est.tht.ini = est.tht0, lambda.ini = lambda0, safe.ini = list(coef = ini$coef, est.tht = ini$est.tht, lambda = ini$lambda),
@@ -382,6 +404,9 @@ frailtyMMpen <- function(formula, data, frailty = "gamma", power = NULL, penalty
       likelihood_all[[z]] = likelihood0
       BIC_all[[z]] = -2*likelihood0 + max(1, log(log(p + 1)))*(sum(abs(coef0) > 1e-6) + 1)*log(b)
       
+      progress_width = floor(z/len_tune*width)
+      cat('[', paste0(c(rep('=', progress_width), '>', rep('-', width - progress_width)),  collapse=''), ']', round(z/len_tune*100),'%\r')
+      
       if (p > N) {
         coef0 = ini$coef
         est.tht0 = ini$est.tht
@@ -389,11 +414,10 @@ frailtyMMpen <- function(formula, data, frailty = "gamma", power = NULL, penalty
       } 
       
       if (sum(abs(coef0)) < 1e-6) {
-        cat(sum(abs(coef0)), "????\n")
+        progress_width = width
+        cat('[', paste0(c(rep('=', progress_width), '>', rep('-', width - progress_width)),  collapse=''), ']', round(100),'%\r')
         break
       }
-      
-      cat(z, "---------\n")
     }
     
     
@@ -427,11 +451,11 @@ frailtyMMpen <- function(formula, data, frailty = "gamma", power = NULL, penalty
     
     initGam = frailtyMMcal(y, X, d, N, a, newid, frailty = "Gamma", power = NULL, penalty = NULL, maxit = 10, threshold = threshold, type = 3)
     
-    # ini = frailtyMMcal(y, X, d, N, a, newid,
-    #                    coef.ini = initGam$coef, est.tht.ini = initGam$est.tht, lambda.ini = initGam$lambda,
-    #                    frailty = frailty, power = power, penalty = NULL, maxit = maxit, threshold = tol, type = 3)
-    ini = initGam
+    if (sum(abs(initGam$coef) > 1e-6) == 0) {
+      initGam = frailtyMMcal(y, X, d, N, a, newid, frailty = frailty, power = NULL, penalty = NULL, maxit = 10, threshold = threshold, type = 3)
+    }
     
+    ini = initGam
     coef0 = ini$coef
     est.tht0 = ini$est.tht
     lambda0 = ini$lambda
@@ -443,6 +467,9 @@ frailtyMMpen <- function(formula, data, frailty = "gamma", power = NULL, penalty
     likelihood_all = list()
     BIC_all = list()
     
+    width <- options()$width/2
+    len_tune = length(tuneseq)
+    
     for (z in seq_len(length(tuneseq))) {
       cur = frailtyMMcal(y, X, d, N, a, newid,
                          coef.ini = coef0, est.tht.ini = est.tht0, lambda.ini = lambda0, safe.ini = list(coef = ini$coef, est.tht = ini$est.tht, lambda = ini$lambda),
@@ -459,6 +486,9 @@ frailtyMMpen <- function(formula, data, frailty = "gamma", power = NULL, penalty
       likelihood_all[[z]] = likelihood0
       BIC_all[[z]] = -2*likelihood0 + max(1, log(log(p + 1)))*(sum(abs(coef0) > 1e-6) + 1)*log(a)
       
+      progress_width = floor(z/len_tune*width)
+      cat('[', paste0(c(rep('=', progress_width), '>', rep('-', width - progress_width)),  collapse=''), ']', round(z/len_tune*100),'%\r')
+      
       if (p > N) {
         coef0 = ini$coef
         est.tht0 = ini$est.tht
@@ -466,11 +496,10 @@ frailtyMMpen <- function(formula, data, frailty = "gamma", power = NULL, penalty
       } 
       
       if (sum(abs(coef0)) < 1e-6) {
-        cat(sum(abs(coef0)), "????\n")
+        progress_width = width
+        cat('[', paste0(c(rep('=', progress_width), '>', rep('-', width - progress_width)),  collapse=''), ']', round(100),'%\r')
         break
       }
-      
-      cat(z, "---------\n")
     }
     
     
